import { ValueOf } from 'next/dist/shared/lib/constants';
import { NsColumnControlProps, sizeObjectType } from './types.d';

export const columnClasses: NsColumnControlProps['columnClass'][] = [
    'ns-col-lg-2-1',
    'ns-col-lg-1-2',
    'ns-col-lg-1-1',
    'ns-col-lg-1-1-1',
    'ns-col-lg-2-1-1',
    'ns-col-lg-1-1-1-1',
    'ns-col-md-1',
    'ns-col-md-1-1',
    'ns-col-md-1-2',
    'ns-col-md-2-1',
    'ns-col-md-1-1-1',
    'ns-col-md-2-1-1',
    'ns-col-sm-1',
    'ns-col-sm-1-1',
    'ns-col-sm-1-2',
    'ns-col-sm-2-1',
];
const sizeKeyPrefix = {
    desktop: 'ns-col-lg-',
    tablet: 'ns-col-md-',
    mobile: 'ns-col-sm-',
} as const;

const sizeKeyMap: Record<ValueOf<typeof sizeKeyPrefix>, string> = {
    [sizeKeyPrefix.desktop]: 'fractions',
    [sizeKeyPrefix.tablet]: 'tabletFractions',
    [sizeKeyPrefix.mobile]: 'mobileFractions',
};

/**
 * getColumnSizes function, we parse the input class and
 * return an object containing optionally containing 3 keys namely [fractions, tabletFractions, mobileFractions]
 * value of each of the key is an array of fractions example ['1','1']
 * meaning that the column should be divided into two fractions each of 1fr
 * @param inputColumnClass = a string that is mix of ns-col-lg-1-1-1 ns-col-md-1-1 ns-col-sm-1 etc
 * @returns {fractions:['1','1','1'], tabletFractions:['1','1'], mobileFractions:['1'] }
 */
export const getColumnSizes = (inputColumnClass: string): sizeObjectType => {
    const columnClass = inputColumnClass.trim();
    const validClasses = columnClass
        .split(' ')
        .filter((e) => Object.values(sizeKeyPrefix).some((prefix) => e.startsWith(prefix)));
    const sizeObject: sizeObjectType = validClasses.reduce((acc: any, current: string) => {
        for (const key of Object.values(sizeKeyPrefix)) {
            if (current.startsWith(key)) {
                const interClass = current.replace(key, '');
                const sizes = interClass.split('-').filter((e) => /^[1-9]$/.test(e));
                acc[sizeKeyMap[key]] = sizes;
                break;
            }
        }
        return acc;
    }, {});
    if (Object.keys(sizeObject).length < 1) {
        /* based on string passed as className, no valid size array could be extracted */
        console.warn(
            'Received invalid className, valid column-control className starts with ' +
                Object.values(sizeKeyPrefix).join(', ')
        );
    }
    return sizeObject;
};
