/* eslint-disable no-unused-vars */
import compose from 'ramda/src/compose';
import find from 'ramda/src/find';
import pathOr from 'ramda/src/pathOr';
import propEq from 'ramda/src/propEq';
import { BynderImageDataType, ImageType } from './types';

export const parseJsonString = (dataString: string) => {
    try {
        return JSON.parse(dataString);
    } catch (error) {
        console.error('error parsing data ', error);
        return {};
    }
};

/** alt text */
export const getBynderAltText = (propKey: string) =>
    compose(pathOr('', ['value']), find(propEq('SEO_Tags', 'name')), pathOr([], [propKey, 0, 'textMetaproperties']));

export const extractBynderDescription = (key: string) => pathOr('', [key, '0', 'description']);

/** end alt text */

/** DAT specific */

export const getBasicDATWidthHeight = (width: number = 900, height: number = 800, transform = 'scale') =>
    `io=transform:${transform},width:${width},height:${height}`;

/** determine whether the image is
     * 1. custom transformation ('customTransformation') ==> further transformation cannot be applied
    https://media.nuskin.com/transform/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1?io=overlay:text,content:abc,font:Open Sans,size:64,color:FFFFFFFF,gravity:center,padding:20,angle:0

    * 2. base transform url ('transformBaseUrl') ==> transformation can be applied in the component
 * "https://media.nuskin.com/transform/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1"

* 3. bynder defined preset ('preset') ==> further transformation cannot be applied
    "https://media.nuskin.com/transform/PDP_Key_Ingredients_Image/3ea706a3-770e-4b66-b112-427ef840f4b6/mail-40th-240207-1"

    * 4. custom preset ('defaultPreset') example: thumbnail, small, mini, original , webImage, ==> further transformation can be applied

    * returns either 'customTransformation' | 'transformBaseUrl' | or preset type of the image example webImage, thumbnail
    * */
export const getImageType = (url: string, fileName: string, files: Record<string, any>) => {
    if (url.includes(`${fileName}?io=`)) {
        return ImageType.customTransformation;
    } else {
        const parts = url.split('/');
        if (parts[3] === 'transform' && parts[5] === fileName) {
            // url?.toLowerCase().includes(`transform/${databaseId}/${fileName}`.toLowerCase()) this check creates error in sometimes url does not have - which is present in database id

            return ImageType.transformBaseUrl;
        } else {
            return retrieveImageKey(files, url);
        }
    }
};
export const retrieveImageKey = (files: Record<string, any>, url: string) => {
    for (const key in files) {
        if (files[key].url === url) {
            return key;
        }
    }
    return ImageType.defaultPreset;
};

export const getUrlBasedOnDerivative = (url: string, imageDerivativeType: string, suffix: string) => {
    if (imageDerivativeType === ImageType.customTransformation) {
        return `${url}&${suffix}`;
    } else if (imageDerivativeType === ImageType.transformBaseUrl) {
        return `${url}?${suffix}`;
    }
    return url;
};

/** end DAT */

/**
 *
 * @param selected : object containing select image url
 * @param isPublic : whether asset is marked public, if it is video and marked public, then mov or gif can be selected otherwise, only webImage can be selected in that case we have to show previewUrl
 * @returns returns selected video or image, if asset is video and is not public, then returns previewUrl
 */
export function getMediaUrl(
    selected: BynderImageDataType['selected'],
    isPublic: boolean,
    previewUrls: BynderImageDataType['previewUrls']
): string {
    const { url = '', type: fileType = '' } = selected ?? {};
    if (fileType === 'VIDEO') {
        if (isPublic) {
            return url;
        } else {
            return previewUrls?.[0] || '';
        }
    }
    return url;
}
