import React from 'react';
import { render, fireEvent, screen } from '@/test-utils/index';
import NsTable from './NsTable';
import { Item } from './types';
import { NsIcon } from '../icon';

const expandableContent = (selectedItem: Item, index: number) => {
    return (
        <div key={index}>
            <p>
                Data: {selectedItem.name} , {selectedItem.calories}
            </p>
            <p>
                Index: {index}
            </p>
            <p>
                Expandable: True
            </p>
        </div>
    )
}

const actionClick = (row: Item) => {
    const handleClick = (event: React.MouseEvent<HTMLButtonElement>) => {
        event.stopPropagation();
        console.log(row)
    }

    return (
        <button onClick={handleClick}>
            <NsIcon
                name='cart'
                size='medium'
            />
        </button>
    )
}

describe('NsTable', () => {
    const headers = [
        { id: 'name', label: 'Name', numeric: false, sortable: true },
        { id: 'age', label: 'Age', numeric: true, sortable: true },
    ];

    const items = [
        { name: 'John', age: 26 },
        { name: 'Jane', age: 32 },
        { name: 'Maria', age: 28 },
        { name: 'Kevin', age: 31 },
        { name: 'Suzan', age: 25 },
        { name: 'Dan', age: 30 },
        { name: 'Micheal', age: 39 },
        { name: 'Sandy', age: 39 },
    ];

    const rows = 5;
    const rowsPerPageOptions = [5, 10, 15];
    const itemKey = 'name';

    it('renders table with correct data', () => {
        render(
            <NsTable
                headers={headers}
                items={items}
                rows={rows}
                rowsPerPageOptions={rowsPerPageOptions}
                itemKey={itemKey}
                isExpandableAll={true}
                loading={true}
                showActions={false}
                showExpandedContent={false}
                expandableContent={expandableContent}
                actionClick={actionClick}
            />
        );

        expect(screen.getByText('Name')).toBeInTheDocument();
        expect(screen.getByText('Age')).toBeInTheDocument();
        expect(screen.getByText('Kevin')).toBeInTheDocument();
        expect(screen.getByText('Maria')).toBeInTheDocument();
        expect(screen.getByText('28')).toBeInTheDocument();
        expect(screen.getByText('31')).toBeInTheDocument();
    });

    it('expandable true click', () => {
        render(
            <NsTable
                headers={headers}
                items={items}
                rows={rows}
                rowsPerPageOptions={rowsPerPageOptions}
                itemKey={itemKey}
                isExpandableAll={true}
                expandableContent={expandableContent}
                actionClick={actionClick}
            />
        );

        
        expect(screen.getByTestId('table-row-0')).toBeInTheDocument();

        const button = screen.getByTestId('table-row-0');

        fireEvent.click(button);
        expect(screen.getByText('Expandable: True')).toBeInTheDocument();
        fireEvent.click(button);
    });

    it('expandable false click', () => {
        render(
            <NsTable
                headers={headers}
                items={items}
                rows={rows}
                rowsPerPageOptions={rowsPerPageOptions}
                itemKey={itemKey}
                isExpandableAll={false}
                expandableContent={expandableContent}
                actionClick={actionClick}
            />
        );

        expect(screen.getByTestId('table-row-0')).toBeInTheDocument();

        const button = screen.getByTestId('table-row-0');

        fireEvent.click(button);
        expect(screen.getByText('Expandable: True')).toBeInTheDocument();
        fireEvent.click(button);
    });

    it('pagination next button click event', async () => {
        render(
            <NsTable
                headers={headers}
                items={items}
                rows={rows}
                rowsPerPageOptions={rowsPerPageOptions}
                itemKey={itemKey}
                isExpandableAll={true}
                expandableContent={expandableContent}
                actionClick={actionClick}
            />
        );

        expect(screen.getByTestId('NavigateNextIcon')).toBeInTheDocument();

        const button = screen.getByTestId('NavigateNextIcon');
        fireEvent.click(button);
        expect(screen.getByText('Sandy')).toBeInTheDocument();
    });

    it('sort rows via age', () => {
        render(
            <NsTable
                headers={headers}
                items={items}
                rows={rows}
                rowsPerPageOptions={rowsPerPageOptions}
                itemKey={itemKey}
                isExpandableAll={true}
                expandableContent={expandableContent}
                actionClick={actionClick}
            />
        );

        expect(screen.getByText('Age')).toBeInTheDocument();
        const button = screen.getByText('Age');
        fireEvent.click(button);

        expect(screen.getByText('John')).toBeInTheDocument();
        fireEvent.click(button);
    });
});
