import { VariantType, closeSnackbar } from 'notistack';
import { render, screen } from '@/test-utils/index';
import React, { PropsWithChildren, useEffect } from 'react';

import { NsToastProvider, ToastOptions, useToast } from '../';

const ToastWrapper = ({ children }: PropsWithChildren<{}>) => {
    return <NsToastProvider>{children}</NsToastProvider>;
};

const ToastComponent = (props: { variant: VariantType; message?: string; options?: ToastOptions }) => {
    const { showToast } = useToast();
    const { variant, message, options } = props;
    useEffect(() => {
        showToast(variant, message, options);
        // eslint-disable-next-line react-hooks/exhaustive-deps
    }, []);
    return <div></div>;
};

describe('Toast', () => {
    afterEach(() => {
        closeSnackbar();
    });
    it('1. should render a toast and displays the message', () => {
        render(
            <ToastWrapper>
                <ToastComponent variant="success" message="Hello" />
            </ToastWrapper>
        );
        expect(screen.queryByText('Hello')).toBeInTheDocument();
    }, 100000);
    it('2. should render a success toast', () => {
        const { container } = render(
            <ToastWrapper>
                <ToastComponent variant="success" message="Hello" />
            </ToastWrapper>
        );
        expect(container.querySelector('.notistack-MuiContent-success')).toBeInTheDocument();
    }, 100000);
    it('3. should render a error toast', () => {
        const { container } = render(
            <ToastWrapper>
                <ToastComponent variant="error" message="Hello" />
            </ToastWrapper>
        );
        expect(container.querySelector('.notistack-MuiContent-error')).toBeInTheDocument();
    }, 100000);
    it('4. should render a info toast', () => {
        const { container } = render(
            <ToastWrapper>
                <ToastComponent variant="info" message="Hello" />
            </ToastWrapper>
        );
        expect(container.querySelector('.notistack-MuiContent-info')).toBeInTheDocument();
    }, 100000);
    it('5. it should render a warning toast', () => {
        const { container } = render(
            <ToastWrapper>
                <ToastComponent variant="warning" message="Hello" />
            </ToastWrapper>
        );
        expect(container.querySelector('.notistack-MuiContent-warning')).toBeInTheDocument();
    }, 100000);
    it('6. should render a warning toast and disappear after 3 seconds', async () => {
        const { container } = render(
            <ToastWrapper>
                <ToastComponent variant="warning" message="Hello" options={{ autoHideDuration: 3000 }} />
            </ToastWrapper>
        );
        expect(container.querySelector('.notistack-MuiContent-warning')).toBeInTheDocument();
        await new Promise((r) => setTimeout(r, 4000));
        expect(container.querySelector('.notistack-MuiContent-warning')).not.toBeInTheDocument();
    }, 100000);
    it('7. should render a action buttons', async () => {
        const { getByTestId } = render(
            <ToastWrapper>
                <ToastComponent
                    variant="warning"
                    message="Hello"
                    options={{
                        actions: [
                            {
                                label: 'Undo',
                                onClick: () => alert('Undo clicked'),
                            },
                        ],
                    }}
                />
            </ToastWrapper>
        );
        expect(getByTestId('action-button')).toBeInTheDocument();
    }, 100000);
    it('8. should render close button', async () => {
        const { getByTestId } = render(
            <ToastWrapper>
                <ToastComponent variant="warning" message="Hello" options={{ showCloseIcon: true }} />
            </ToastWrapper>
        );
        expect(getByTestId('icon')).toBeInTheDocument();
    }, 100000);
    it('9. should render a action buttons and close button', async () => {
        const { getAllByRole } = render(
            <ToastWrapper>
                <ToastComponent
                    variant="warning"
                    message="Hello"
                    options={{
                        actions: [
                            {
                                label: 'Undo',
                                onClick: () => alert('Undo clicked'),
                            },
                        ],
                        showCloseIcon: true,
                    }}
                />
            </ToastWrapper>
        );
        expect(getAllByRole('button')?.length).toBe(2);
    }, 100000);
});
